@@ -58,7 +58,9 @@ enum
   ARG_DROP_PROBABILITY,
   ARG_SILENT,
   ARG_LAST_MESSAGE,
-  ARG_DUMP
+  ARG_DUMP,
+  ARG_SYNC,
+  ARG_CHECK_PERFECT
 };
 
 
@@ -68,12 +70,15 @@ enum
 GST_BOILERPLATE_FULL (GstIdentity, gst_identity, GstElement, GST_TYPE_ELEMENT,
     _do_init);
 
+static void gst_identity_finalize (GObject * object);
 static void gst_identity_set_property (GObject * object, guint prop_id,
     const GValue * value, GParamSpec * pspec);
 static void gst_identity_get_property (GObject * object, guint prop_id,
     GValue * value, GParamSpec * pspec);
 
 static void gst_identity_chain (GstPad * pad, GstData * _data);
+static void gst_identity_set_clock (GstElement * element, GstClock * clock);
+
 
 static guint gst_identity_signals[LAST_SIGNAL] = { 0 };
 
@@ -84,13 +89,27 @@ gst_identity_base_init (gpointer g_class)
 
   gst_element_class_set_details (gstelement_class, &gst_identity_details);
 }
+
+static void
+gst_identity_finalize (GObject * object)
+{
+  GstIdentity *identity;
+
+  identity = GST_IDENTITY (object);
+
+  g_free (identity->last_message);
+
+  G_OBJECT_CLASS (parent_class)->finalize (object);
+}
+
 static void
 gst_identity_class_init (GstIdentityClass * klass)
 {
   GObjectClass *gobject_class;
+  GstElementClass *gstelement_class;
 
   gobject_class = G_OBJECT_CLASS (klass);
-
+  gstelement_class = GST_ELEMENT_CLASS (klass);
 
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_LOOP_BASED,
       g_param_spec_boolean ("loop-based", "Loop-based",
@@ -119,14 +138,25 @@ gst_identity_class_init (GstIdentityClass * klass)
   g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_DUMP,
       g_param_spec_boolean ("dump", "Dump", "Dump buffer contents", FALSE,
           G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_SYNC,
+      g_param_spec_boolean ("sync", "Synchronize", "Synchronize to pipeline clock", FALSE,
+          G_PARAM_READWRITE));
+  g_object_class_install_property (G_OBJECT_CLASS (klass), ARG_CHECK_PERFECT,
+      g_param_spec_boolean ("check-perfect", "Check For Perfect Stream",
+          "Verify that the stream is time- and data-contiguous", FALSE,
+          G_PARAM_READWRITE));
 
   gst_identity_signals[SIGNAL_HANDOFF] =
       g_signal_new ("handoff", G_TYPE_FROM_CLASS (klass), G_SIGNAL_RUN_LAST,
       G_STRUCT_OFFSET (GstIdentityClass, handoff), NULL, NULL,
       gst_marshal_VOID__POINTER, G_TYPE_NONE, 1, GST_TYPE_BUFFER);
 
+  gobject_class->finalize = GST_DEBUG_FUNCPTR (gst_identity_finalize);
   gobject_class->set_property = GST_DEBUG_FUNCPTR (gst_identity_set_property);
   gobject_class->get_property = GST_DEBUG_FUNCPTR (gst_identity_get_property);
+
+  gstelement_class->set_clock = GST_DEBUG_FUNCPTR (gst_identity_set_clock);
+
 }
 
 static void
@@ -150,11 +180,27 @@ gst_identity_init (GstIdentity * identity)
   identity->error_after = -1;
   identity->drop_probability = 0.0;
   identity->silent = FALSE;
+  identity->sync = FALSE;
+  identity->check_perfect = FALSE;
+  identity->prev_timestamp = GST_CLOCK_TIME_NONE;
+  identity->prev_duration = GST_CLOCK_TIME_NONE;
+  identity->prev_offset_end = -1;
   identity->dump = FALSE;
   identity->last_message = NULL;
   identity->srccaps = NULL;
+
+  GST_FLAG_SET (identity, GST_ELEMENT_EVENT_AWARE);
 }
 
+static void
+gst_identity_set_clock (GstElement * element, GstClock * clock)
+{
+  GstIdentity *identity = GST_IDENTITY (element);
+
+  gst_object_replace ((GstObject **) & identity->clock, (GstObject *) clock);
+}
+
+
 static void
 gst_identity_chain (GstPad * pad, GstData * _data)
 {
@@ -168,6 +214,51 @@ gst_identity_chain (GstPad * pad, GstData * _data)
 
   identity = GST_IDENTITY (gst_pad_get_parent (pad));
 
+  if (GST_IS_EVENT (buf)) {
+    GstEvent *event = GST_EVENT (buf);
+
+    if (!identity->silent) {
+      g_free (identity->last_message);
+
+      identity->last_message =
+          g_strdup_printf ("chain   ******* (%s:%s)E (type: %d) %p",
+          GST_DEBUG_PAD_NAME (pad), GST_EVENT_TYPE (event), event);
+
+      g_object_notify (G_OBJECT (identity), "last_message");
+    }
+    gst_pad_event_default (pad, event);
+    return;
+  }
+
+  /* see if we need to do perfect stream checking */
+    /* invalid timestamp drops us out of check.  FIXME: maybe warn ? */
+  if (identity->check_perfect &&
+      GST_BUFFER_TIMESTAMP (buf) != GST_CLOCK_TIME_NONE)
+  {
+    /* check if we had a previous buffer to compare to */
+    if (identity->prev_timestamp != GST_CLOCK_TIME_NONE) {
+      if (identity->prev_timestamp + identity->prev_duration != GST_BUFFER_TIMESTAMP (buf)) {
+        GST_WARNING_OBJECT (identity,
+            "Buffer not time-contiguous with previous one: "
+            "prev ts %" GST_TIME_FORMAT ", prev dur %" GST_TIME_FORMAT
+            ", new ts %" GST_TIME_FORMAT,
+            GST_TIME_ARGS (identity->prev_timestamp),
+            GST_TIME_ARGS (identity->prev_duration),
+            GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)));
+      }
+      if (identity->prev_offset_end != GST_BUFFER_OFFSET (buf)) {
+        GST_WARNING_OBJECT (identity,
+            "Buffer not data-contiguous with previous one: "
+            "prev offset_end %" G_GINT64_FORMAT ", new offset %" G_GINT64_FORMAT,
+            identity->prev_offset_end, GST_BUFFER_OFFSET (buf));
+      }
+    }
+    /* update prev values */
+    identity->prev_timestamp = GST_BUFFER_TIMESTAMP (buf);
+    identity->prev_duration = GST_BUFFER_DURATION (buf);
+    identity->prev_offset_end = GST_BUFFER_OFFSET_END (buf);
+  }
+
   if (identity->error_after >= 0) {
     identity->error_after--;
     if (identity->error_after == 0) {
@@ -180,13 +271,15 @@ gst_identity_chain (GstPad * pad, GstData * _data)
 
   if (identity->drop_probability > 0.0) {
     if ((gfloat) (1.0 * rand () / (RAND_MAX)) < identity->drop_probability) {
-      if (identity->last_message != NULL) {
-        g_free (identity->last_message);
-      }
+      g_free (identity->last_message);
       identity->last_message =
-          g_strdup_printf ("dropping   ******* (%s:%s)i (%d bytes, %"
-          G_GINT64_FORMAT ")", GST_DEBUG_PAD_NAME (identity->sinkpad),
-          GST_BUFFER_SIZE (buf), GST_BUFFER_TIMESTAMP (buf));
+          g_strdup_printf ("dropping   ******* (%s:%s)i (%d bytes, timestamp: %"
+          GST_TIME_FORMAT ", duration: %" GST_TIME_FORMAT ", offset: %"
+          G_GINT64_FORMAT ", offset_end: % " G_GINT64_FORMAT ", flags: %d) %p",
+          GST_DEBUG_PAD_NAME (identity->sinkpad), GST_BUFFER_SIZE (buf),
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (buf)), GST_BUFFER_OFFSET (buf),
+          GST_BUFFER_OFFSET_END (buf), GST_BUFFER_FLAGS (buf), buf);
       g_object_notify (G_OBJECT (identity), "last-message");
       gst_buffer_unref (buf);
       return;
@@ -200,9 +293,13 @@ gst_identity_chain (GstPad * pad, GstData * _data)
     if (!identity->silent) {
       g_free (identity->last_message);
       identity->last_message =
-          g_strdup_printf ("chain   ******* (%s:%s)i (%d bytes, %"
-          G_GINT64_FORMAT ")", GST_DEBUG_PAD_NAME (identity->sinkpad),
-          GST_BUFFER_SIZE (buf), GST_BUFFER_TIMESTAMP (buf));
+          g_strdup_printf ("chain   ******* (%s:%s)i (%d bytes, timestamp: %"
+          GST_TIME_FORMAT ", duration: %" GST_TIME_FORMAT ", offset: %"
+          G_GINT64_FORMAT ", offset_end: % " G_GINT64_FORMAT ", flags: %d) %p",
+          GST_DEBUG_PAD_NAME (identity->sinkpad), GST_BUFFER_SIZE (buf),
+          GST_TIME_ARGS (GST_BUFFER_TIMESTAMP (buf)),
+          GST_TIME_ARGS (GST_BUFFER_DURATION (buf)), GST_BUFFER_OFFSET (buf),
+          GST_BUFFER_OFFSET_END (buf), GST_BUFFER_FLAGS (buf), buf);
       g_object_notify (G_OBJECT (identity), "last-message");
     }
 
@@ -212,6 +309,11 @@ gst_identity_chain (GstPad * pad, GstData * _data)
     if (i > 1)
       gst_buffer_ref (buf);
 
+    if (identity->sync) {
+      if (identity->clock) {
+        gst_element_wait (GST_ELEMENT (identity), GST_BUFFER_TIMESTAMP (buf));
+      }
+    }
     gst_pad_push (identity->srcpad, GST_DATA (buf));
 
     if (identity->sleep_time)
@@ -285,7 +387,14 @@ gst_identity_set_property (GObject * object, guint prop_id,
     case ARG_DROP_PROBABILITY:
       identity->drop_probability = g_value_get_float (value);
       break;
+    case ARG_SYNC:
+      identity->sync = g_value_get_boolean (value);
+      break;
+    case ARG_CHECK_PERFECT:
+      identity->check_perfect = g_value_get_boolean (value);
+      break;
     default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
   }
 }
@@ -326,6 +435,12 @@ gst_identity_get_property (GObject * object, guint prop_id, GValue * value,
     case ARG_LAST_MESSAGE:
       g_value_set_string (value, identity->last_message);
       break;
+    case ARG_SYNC:
+      g_value_set_boolean (value, identity->sync);
+      break;
+    case ARG_CHECK_PERFECT:
+      g_value_set_boolean (value, identity->check_perfect);
+      break;
     default:
       G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
       break;
