@@ -66,7 +66,8 @@ typedef struct
   GstIndexResolverMethod method;
   GstIndexResolver resolver;
   gpointer user_data;
-} ResolverEntry;
+}
+ResolverEntry;
 
 static const ResolverEntry resolvers[] = {
   {GST_INDEX_RESOLVER_CUSTOM, NULL, NULL},
@@ -88,6 +89,7 @@ gst_index_resolver_get_type (void)
         "Resolve an object to its path in the pipeline"},
     {0, NULL, NULL},
   };
+
   if (!index_resolver_type) {
     index_resolver_type =
         g_enum_register_static ("GstIndexResolver", index_resolver);
@@ -123,10 +125,11 @@ gst_index_get_type (void)
       NULL,
       NULL,
       sizeof (GstIndex),
-      1,
+      0,
       (GInstanceInitFunc) gst_index_init,
       NULL
     };
+
     index_type =
         g_type_register_static (GST_TYPE_OBJECT, "GstIndex", &index_info, 0);
   }
@@ -409,6 +412,8 @@ gst_index_set_resolver (GstIndex * index,
  * @entry: the entry to copy
  *
  * Copies an entry and returns the result.
+ *
+ * Returns: a newly allocated #GstIndexEntry.
  */
 GstIndexEntry *
 gst_index_entry_copy (GstIndexEntry * entry)
@@ -736,7 +741,11 @@ gst_index_add_object (GstIndex * index, gint id, gchar * key,
 static gint
 gst_index_compare_func (gconstpointer a, gconstpointer b, gpointer user_data)
 {
-  return (gint) a - (gint) b;
+  if (a < b)
+    return -1;
+  if (a > b)
+    return 1;
+  return 0;
 }
 
 /**
@@ -860,6 +869,7 @@ gst_index_factory_get_type (void)
       (GInstanceInitFunc) gst_index_factory_init,
       NULL
     };
+
     indexfactory_type = g_type_register_static (GST_TYPE_PLUGIN_FEATURE,
         "GstIndexFactory", &indexfactory_info, 0);
   }
