@@ -80,9 +80,8 @@ struct _GstPlugin {
   gpointer _gst_reserved[GST_PADDING];
 };
 
-#ifndef GST_PLUGIN_STATIC				
-#define GST_PLUGIN_DEFINE_DYNAMIC(major,minor,name,description,init,version,license,package,origin)	\
-GstPluginDesc gst_plugin_desc = {		      	\
+#define GST_PLUGIN_DEFINE(major,minor,name,description,init,version,license,package,origin)	\
+GST_PLUGIN_EXPORT GstPluginDesc gst_plugin_desc = {		      	\
   major,						\
   minor,						\
   name,							\
@@ -93,11 +92,9 @@ GstPluginDesc gst_plugin_desc = {		      	\
   license,					      	\
   package,						\
   origin,						\
-  GST_PADDING_INIT                                      \
-};							
-#define GST_PLUGIN_DEFINE_STATIC(major,minor,name,description,init,version,license,package,origin)
-#else
-#define GST_PLUGIN_DEFINE_DYNAMIC(major,minor,name,description,init,version,license,package,origin)
+  GST_PADDING_INIT				        \
+};
+
 #define GST_PLUGIN_DEFINE_STATIC(major,minor,name,description,init,version,license,package,origin)  \
 static void GST_GNUC_CONSTRUCTOR			\
 _gst_plugin_static_init__ ##init (void)			\
@@ -117,12 +114,7 @@ _gst_plugin_static_init__ ##init (void)			\
   };							\
   _gst_plugin_register_static (&plugin_desc_);		\
 }			
-#endif
 
-#define GST_PLUGIN_DEFINE(major,minor,name,description,init,version,license,package,origin)\
-  GST_PLUGIN_DEFINE_STATIC(major,minor,name,description,init,version,license,package,origin)\
-  GST_PLUGIN_DEFINE_DYNAMIC(major,minor,name,description,init,version,license,package,origin)
-  
 #define GST_LICENSE_UNKNOWN "unknown"
 
 
@@ -136,8 +128,7 @@ void			_gst_plugin_initialize		(void);
 void 			_gst_plugin_register_static 	(GstPluginDesc *desc);
 
 G_CONST_RETURN gchar*	gst_plugin_get_name		(GstPlugin *plugin);
-void			gst_plugin_set_name		(GstPlugin *plugin, const gchar *name);
-G_CONST_RETURN gchar*	gst_plugin_get_longname		(GstPlugin *plugin);
+G_CONST_RETURN gchar*	gst_plugin_get_description	(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_filename		(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_license		(GstPlugin *plugin);
 G_CONST_RETURN gchar*	gst_plugin_get_package		(GstPlugin *plugin);
@@ -158,6 +149,7 @@ gboolean		gst_plugin_name_filter		(GstPlugin *plugin, const gchar *name);
 GList*			gst_plugin_get_feature_list	(GstPlugin *plugin);
 GstPluginFeature*	gst_plugin_find_feature		(GstPlugin *plugin, const gchar *name, GType type);
 
+gboolean 		gst_plugin_check_file		(const gchar *filename, GError** error);
 GstPlugin * 		gst_plugin_load_file		(const gchar *filename, GError** error);
 gboolean 		gst_plugin_unload_plugin	(GstPlugin *plugin);
 
