@@ -20,6 +20,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 #endif
+#include <math.h>
 #include <stdlib.h>
 #include <string.h>
 #include <ctype.h>
@@ -869,7 +870,7 @@ gst_value_serialize_buffer (const GValue * value)
   data = GST_BUFFER_DATA (buffer);
   size = GST_BUFFER_SIZE (buffer);
 
-  string = malloc (size * 2 + 1);
+  string = g_malloc (size * 2 + 1);
   for (i = 0; i < size; i++) {
     sprintf (string + i * 2, "%02x", data[i]);
   }
@@ -1254,7 +1255,7 @@ gst_string_wrap (const char *s)
   }
 
   if (!wrap)
-    return strdup (s);
+    return g_strdup (s);
 
   e = d = g_malloc (len + 3);
 
@@ -1525,6 +1526,32 @@ gst_value_intersect_list (GValue * dest, const GValue * value1,
   return ret;
 }
 
+static gboolean
+gst_value_intersect_fixed_list (GValue * dest, const GValue * src1,
+    const GValue * src2)
+{
+  gint size, n;
+  GValue val = { 0 };
+
+  /* only works on similar-sized arrays */
+  size = gst_value_list_get_size (src1);
+  if (size != gst_value_list_get_size (src2))
+    return FALSE;
+  g_value_init (dest, GST_TYPE_FIXED_LIST);
+
+  for (n = 0; n < size; n++) {
+    if (!gst_value_intersect (&val, gst_value_list_get_value (src1, n),
+            gst_value_list_get_value (src2, n))) {
+      g_value_unset (dest);
+      return FALSE;
+    }
+    gst_value_list_append_value (dest, &val);
+    g_value_unset (&val);
+  }
+
+  return TRUE;
+}
+
 /***************
  * subtraction *
  ***************/
@@ -2496,7 +2523,7 @@ gst_value_deserialize_fraction (GValue * dest, const char *s)
     return FALSE;
   tmp = g_strndup (s, (size_t) (div - s));
   num = atoi (tmp);
-  free (tmp);
+  g_free (tmp);
   den = atoi (div + 1);
 
   gst_value_set_fraction (dest, num, den);
@@ -2518,6 +2545,95 @@ gst_value_transform_string_fraction (const GValue * src_value,
   gst_value_deserialize_fraction (dest_value, src_value->data[0].v_pointer);
 }
 
+#define MAX_TERMS       30
+#define MIN_DIVISOR     1.0e-10
+#define MAX_ERROR       1.0e-20
+
+/* use continued fractions to transform a double into a fraction,
+ * see http://mathforum.org/dr.math/faq/faq.fractions.html#decfrac.
+ * This algorithm takes care of overflows.
+ */
+static void
+gst_value_transform_double_fraction (const GValue * src_value,
+    GValue * dest_value)
+{
+  gdouble V, F;                 /* double being converted */
+  gint N, D;                    /* will contain the result */
+  gint A;                       /* current term in continued fraction */
+  gint64 N1, D1;                /* numerator, denominator of last approx */
+  gint64 N2, D2;                /* numerator, denominator of previous approx */
+  gint i;
+  gboolean negative = FALSE;
+
+  /* initialize fraction being converted */
+  F = src_value->data[0].v_double;
+  if (F < 0.0) {
+    F = -F;
+    negative = TRUE;
+  }
+
+  V = F;
+  /* initialize fractions with 1/0, 0/1 */
+  N1 = 1;
+  D1 = 0;
+  N2 = 0;
+  D2 = 1;
+  N = 1;
+  D = 1;
+
+  for (i = 0; i < MAX_TERMS; i++) {
+    /* get next term */
+    A = floor (F);
+    /* get new divisor */
+    F = F - A;
+
+    /* calculate new fraction in temp */
+    N2 = N1 * A + N2;
+    D2 = D1 * A + D2;
+
+    /* guard against overflow */
+    if (N2 > G_MAXINT || D2 > G_MAXINT) {
+      break;
+    }
+
+    N = N2;
+    D = D2;
+
+    /* save last two fractions */
+    N2 = N1;
+    D2 = D1;
+    N1 = N;
+    D1 = D;
+
+    /* quit if dividing by zero or close enough to target */
+    if (F < MIN_DIVISOR || fabs (V - ((gdouble) N) / D) < MAX_ERROR) {
+      break;
+    }
+
+    /* Take reciprocal */
+    F = 1 / F;
+  }
+  /* fix for overflow */
+  if (D == 0) {
+    N = G_MAXINT;
+    D = 1;
+  }
+  /* fix for negative */
+  if (negative)
+    N = -N;
+
+  /* will also simplify */
+  gst_value_set_fraction (dest_value, N, D);
+}
+
+static void
+gst_value_transform_fraction_double (const GValue * src_value,
+    GValue * dest_value)
+{
+  dest_value->data[0].v_double = ((double) src_value->data[0].v_int) /
+      ((double) src_value->data[1].v_int);
+}
+
 static int
 gst_value_compare_fraction (const GValue * value1, const GValue * value2)
 {
@@ -2791,6 +2907,10 @@ _gst_value_initialize (void)
       gst_value_transform_fraction_string);
   g_value_register_transform_func (G_TYPE_STRING, GST_TYPE_FRACTION,
       gst_value_transform_string_fraction);
+  g_value_register_transform_func (GST_TYPE_FRACTION, G_TYPE_DOUBLE,
+      gst_value_transform_fraction_double);
+  g_value_register_transform_func (G_TYPE_DOUBLE, GST_TYPE_FRACTION,
+      gst_value_transform_double_fraction);
 
   gst_value_register_intersect_func (G_TYPE_INT, GST_TYPE_INT_RANGE,
       gst_value_intersect_int_int_range);
@@ -2800,6 +2920,8 @@ _gst_value_initialize (void)
       gst_value_intersect_double_double_range);
   gst_value_register_intersect_func (GST_TYPE_DOUBLE_RANGE,
       GST_TYPE_DOUBLE_RANGE, gst_value_intersect_double_range_double_range);
+  gst_value_register_intersect_func (GST_TYPE_FIXED_LIST,
+      GST_TYPE_FIXED_LIST, gst_value_intersect_fixed_list);
 
   gst_value_register_subtract_func (G_TYPE_INT, GST_TYPE_INT_RANGE,
       gst_value_subtract_int_int_range);
